﻿-- Created by Steve Stedman
--    http://SteveStedman.com
--    http://StedmanSolutions.com
--    twitter:   @SqlEmt


RAISERROR ('Dont run the whole script, just run sections at a time', 20, 1)  WITH LOG



USE master;
GO
ALTER DATABASE CorruptionChallenge6 SET SINGLE_USER WITH ROLLBACK IMMEDIATE;


RESTORE DATABASE [CorruptionChallenge6] 
   FROM DISK = N'C:\DBBackups\CorruptionChallenge6.bak' 
   WITH FILE = 1,  
        MOVE N'CorruptionChallenge6' TO N'C:\SQL_DATA\CorruptionChallenge6.mdf',  
        MOVE N'CorruptionChallenge6_log' TO N'C:\SQL_DATA\CorruptionChallenge6_log.ldf',  
        RECOVERY, REPLACE;

-- restore a second copy that I can use to compare against as I fix the first.
-- I only know to do this becuase the first time through the process I discovered I needed it.
RESTORE DATABASE [CorruptionChallenge6_copy] 
   FROM DISK = N'C:\DBBackups\CorruptionChallenge6.bak' 
   WITH FILE = 1,  
        MOVE N'CorruptionChallenge6' TO N'C:\SQL_DATA\CorruptionChallenge6_copy.mdf',  
        MOVE N'CorruptionChallenge6_log' TO N'C:\SQL_DATA\CorruptionChallenge6_copy_log.ldf',  
        RECOVERY, REPLACE;

USE CorruptionChallenge6 ;
GO
-- now lets figure out what is wrong...
DBCC CheckDB(CorruptionChallenge6) WITH NO_INFOMSGS;

DBCC CheckTable(Customers) WITH NO_INFOMSGS;

SELECT * 
  FROM [Customers];

-- check what columns are covered by indexes.
SELECT ind.name as IndexName, col.name as ColumnName, ic.is_included_column
  FROM sys.indexes ind 
 INNER JOIN sys.index_columns ic ON  ind.object_id = ic.object_id and ind.index_id = ic.index_id 
 INNER JOIN sys.columns col ON ic.object_id = col.object_id and ic.column_id = col.column_id 
 WHERE ind.object_id = OBJECT_ID('Customers') 
   AND ind.is_unique = 0 
   AND ind.is_unique_constraint = 0 
 ORDER BY ind.name, ind.index_id, ic.index_column_id;


SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=PK_Customers)
EXCEPT
SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName]);


SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName])
EXCEPT
SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=PK_Customers);


DBCC CheckTable(Customers) WITH NO_INFOMSGS;





ALTER DATABASE CorruptionChallenge6 SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
-- first try to rebuild
BEGIN TRANSACTION; -- so we can undo if we need to

	DBCC CheckTable([Customers], REPAIR_REBUILD) WITH NO_INFOMSGS;

	-- no good, nothing changed
	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=PK_Customers)
	EXCEPT
	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName]);


	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName])
	EXCEPT
	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=PK_Customers);
ROLLBACK TRANSACTION;




BEGIN TRANSACTION; -- so we can undo if we need to

	-- the following causes parts of the table to be lost.
	DBCC CheckTable([Customers], REPAIR_ALLOW_DATA_LOSS) WITH NO_INFOMSGS;

	-- check to be sure its all fixed.
	DBCC CheckDB(CorruptionChallenge6) WITH NO_INFOMSGS;

	-- compare table to the indexes - nothing mismatched	
	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=PK_Customers)
	EXCEPT
	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName]);

	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName])
	EXCEPT
	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=PK_Customers);
    -- since the match it tells me that we have new indexes that include the corrupt rows.


	-- compare table to the backed up copy indexes.
	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName])
	EXCEPT
	SELECT id, FirstName, LastName FROM [CorruptionChallenge6_copy].[dbo].[Customers] WITH (INDEX=[ncFName]);
	--252971	€ea	MULLEN
	--992	Aliwander	FISCHER

	SELECT id, FirstName, LastName FROM [CorruptionChallenge6_copy].[dbo].[Customers] WITH (INDEX=[ncFName])
	EXCEPT
	SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName]);
	--992	Alexander	FISCHER
	--252971	Mia	MULLEN

	
ROLLBACK TRANSACTION;
--COMMIT TRANSACTION;






SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName])
EXCEPT
SELECT id, FirstName, LastName FROM [CorruptionChallenge6_copy].[dbo].[Customers] WITH (INDEX=[ncFName]);
--252971	€ea	MULLEN
--992	Aliwander	FISCHER

SELECT id, FirstName, LastName FROM [CorruptionChallenge6_copy].[dbo].[Customers] WITH (INDEX=[ncFName])
EXCEPT
SELECT id, FirstName, LastName FROM [Customers] WITH (INDEX=[ncFName]);
--992	Alexander	FISCHER
--252971	Mia	MULLEN


SELECT * FROM [CorruptionChallenge6_copy].[dbo].[Customers] 
EXCEPT
SELECT * FROM [Customers] ;

SELECT * FROM [Customers] 
EXCEPT
SELECT * FROM [CorruptionChallenge6_copy].[dbo].[Customers];





SELECT FirstName, count(1)
  FROM [Customers]
 GROUP BY FirstName
ORDER by 2 asc;

SELECT MiddleName, count(1)
  FROM [Customers]
 GROUP BY MiddleName
ORDER by 2 asc;

SELECT LastName, count(1)
  FROM [Customers]
 GROUP BY LastName
ORDER by 2 asc;


SELECT COUNT(1) AS check1,
       COUNT(DISTINCT FirstName) AS check2,
       COUNT(DISTINCT MiddleName) AS check3,
       COUNT(DISTINCT LastName) AS check4,
	   CHECKSUM_AGG(CHECKSUM(FirstName)) AS check5,
	   CHECKSUM_AGG(CHECKSUM(MiddleName)) AS check6,
	   CHECKSUM_AGG(CHECKSUM(LastName)) AS check7
  FROM [Customers];


UPDATE [Customers] 
SET FirstName = 'Alexander' 
WHERE id = 992;


UPDATE [Customers] 
SET FirstName = 'Mia' 
WHERE id = 252971;


SELECT COUNT(1) AS check1,
       COUNT(DISTINCT FirstName) AS check2,
       COUNT(DISTINCT MiddleName) AS check3,
       COUNT(DISTINCT LastName) AS check4,
	   CHECKSUM_AGG(CHECKSUM(FirstName)) AS check5,
	   CHECKSUM_AGG(CHECKSUM(MiddleName)) AS check6,
	   CHECKSUM_AGG(CHECKSUM(LastName)) AS check7
  FROM [Customers];


--check1	check2	check3	check4	check5	check6	check7
--254031	17	17	879	1046491532	179	1330080960
